==> version4_v4.0.4_lgpl.tgz: Version4_lgpl/Donjon/src/MACINI.f <==
*DECK MACINI
      SUBROUTINE MACINI(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* construct a new macrolib that will contain one mixture number per
* material region; fuel-map macrolib is required.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal.
*
*Author(s): J. Koclas, E. Varin, D. Sekki
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): create type(L_MACROLIB);
*         HENTRY(2): modification type(L_MATEX);
*         HENTRY(3): read-only type(L_MACROLIB);
*         HENTRY(4): read-only type(L_MACROLIB) (optional).
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NENTRY
      CHARACTER HENTRY(NENTRY)*12
      INTEGER IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40,IOUT=6)
      CHARACTER HSIGN*12,TEXT*12
      INTEGER IBASE(1),KCHAR(3),ISTATE(NSTATE)
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
      DOUBLE PRECISION DFLOT
      LOGICAL LMAP
*----
*  PARAMETER VALIDATION
*----
      IF((NENTRY.LE.2).OR.(NENTRY.GE.5)) 
     1  CALL XABORT('@MACINI: 4 PARAMETERS EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2))CALL XABORT('@RMA'
     1 //'C: LCM OBJECT EXPECTED AT LHS.')
      IF(JENTRY(1).NE.0)CALL XABORT('@MACINI: CREATE MODE EXPECTED'
     1 //' FOR L_MACROLIB AT LHS.')
      IPMAC=KENTRY(1)
*     L_MATEX
      IF((IENTRY(2).NE.1).AND.(IENTRY(2).NE.2))CALL XABORT('@RMA'
     1 //'C: LCM OBJECT EXPECTED AT RHS.')
      IF(JENTRY(2).NE.1)CALL XABORT('@MACINI: MODIFICATION MODE EX'
     1 //'PECTED FOR L_MATEX OBJECT.')
      CALL LCMGET(KENTRY(2),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
      IF(HSIGN.NE.'L_MATEX')THEN
        TEXT=HENTRY(2)
        CALL XABORT('@MACINI: SIGNATURE OF '//TEXT//' IS '//HSIGN//
     1  '. L_MATEX EXPECTED AT RHS.')
      ENDIF
      IPMTX=KENTRY(2)
      DO IEN=3,NENTRY
      IF((IENTRY(IEN).NE.1).AND.(IENTRY(IEN).NE.2))CALL XABORT('@R'
     1 //'MAC: LCM OBJECT EXPECTED AT RHS.')
      IF(JENTRY(IEN).NE.2)CALL XABORT('@MACINI: READ-ONLY MODE EXPEC'
     1 //'TED FOR THE LCM OBJECTS AT RHS.')
      ENDDO
*     L_MACROLIB(1)
      CALL LCMGET(KENTRY(3),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
      IF(HSIGN.NE.'L_MACROLIB')THEN
        TEXT=HENTRY(3)
        CALL XABORT('@MACINI: SIGNATURE OF '//TEXT//' IS '//HSIGN//
     1  '. FIRST MACROLIB EXPECTED AT RHS.')
      ENDIF
      IPMAC1=KENTRY(3)
*     L_MACROLIB(2)
      IF(NENTRY.EQ.4) THEN
      	CALL LCMGET(KENTRY(4),'SIGNATURE',KCHAR)
      	WRITE(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
      	IF(HSIGN.NE.'L_MACROLIB')THEN
          TEXT=HENTRY(4)
          CALL XABORT('@MACINI: SIGNATURE OF '//TEXT//' IS '//HSIGN//
     1    '. FUEL-MAP MACROLIB EXPECTED AT RHS.')
      	ENDIF
      	IPMAC2=KENTRY(4)
      ELSE
      	IPMAC2=0
      ENDIF
*----
*  RECOVER INFORMATION
*----
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPMAC1,'STATE-VECTOR',ISTATE)
*     MACROLIB(1)-INFO
      NGRP=ISTATE(1)
      NMIX1=ISTATE(2)
      NL=ISTATE(3)
      LEAK=ISTATE(9)
*     MACROLIB(2)-INFO
      IF(NENTRY.EQ.4) THEN
      	CALL XDISET(ISTATE,NSTATE,0)
      	CALL LCMGET(IPMAC2,'STATE-VECTOR',ISTATE)
      	NMIX2=ISTATE(2)
        IF(ISTATE(1).NE.NGRP)CALL XABORT('@MACINI: DIFFERENT NGR'
     1 //'P NUMBER IN TWO MACROLIB OBJECTS.')
      	IF(ISTATE(3).NE.NL)CALL XABORT('@MACINI: DIFFERENT NL NU'
     1 //'MBER IN TWO MACROLIB OBJECTS.')
      	IF(ISTATE(9).NE.LEAK)CALL XABORT('@MACINI: DIFFERENT LEAK'
     1 //' NUMBER IN TWO MACROLIB OBJECTS.')
      ENDIF
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPMTX,'STATE-VECTOR',ISTATE)
*     MATEX-INFO
      NMIX=ISTATE(2)
      NTOT=ISTATE(5)
      CALL SETARA(IBASE,NTOT,IMIX)
      CALL XDISET(IBASE(IMIX),NTOT,0)
      CALL LCMGET(IPMTX,'MAT',IBASE(IMIX))
      IMPX=1
      CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(ITYP.NE.3)CALL XABORT('@MACINI: CHARACTER DATA EXPECTED.')
      IF(TEXT.NE.'EDIT')GOTO 10
*     READ PRINTING INDEX
      CALL REDGET(ITYP,IMPX,FLOT,TEXT,DFLOT)
      IF(ITYP.NE.1)CALL XABORT('@MACINI: INTEGER FOR EDIT EXPECTED.')
      CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
   10 IF(TEXT.NE.';')CALL XABORT('@MACINI: FINAL ; EXPECTED.')
*----
*  NEW MACROLIB CREATION
*----
      IF(IMPX.GT.1)WRITE(IOUT,*)'NUMBER OF ENERGY GROUPS  ',NGRP
      IF(IMPX.GT.1)WRITE(IOUT,*)'TOTAL NUMBER OF MIXTURES ',NMIX
*     DO NOT INCLUDE FUEL PROPERTIES
      LMAP=.FALSE.
      IF(IMPX.GT.0)WRITE(IOUT,*)'** TREATING FIRST MACROLIB **'
      CALL MACCRE(IPMAC1,IPMAC,NL,NGRP,NMIX1,NMIX,NTOT,
     1            IBASE(IMIX),LMAP,IMPX)
      IF(IMPX.GT.1)CALL LCMLIB(IPMAC)
*     INCLUDE FUEL PROPERTIES
      IF(NENTRY.EQ.4) THEN
      	LMAP=.TRUE.
      	IF(IMPX.GT.0)WRITE(IOUT,*)'** TREATING FUEL-MAP MACROLIB **'
      	CALL MACCRE(IPMAC2,IPMAC,NL,NGRP,NMIX2,NMIX,NTOT,
     1            IBASE(IMIX),LMAP,IMPX)
      	CALL RLSARA(IBASE(IMIX))
      ENDIF
*----
*  STATE-VECTOR
*----
      CALL XDISET(ISTATE,NSTATE,0)
      ISTATE(1)=NGRP
      ISTATE(2)=NMIX
      ISTATE(3)=NL
      ISTATE(4)=1
      ISTATE(9)=LEAK
      CALL LCMPUT(IPMAC,'STATE-VECTOR',NSTATE,1,ISTATE)
      HSIGN='L_MACROLIB'
      READ(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
      CALL LCMPUT(IPMAC,'SIGNATURE',3,3,KCHAR)
      IF(IMPX.GT.0)CALL LCMLIB(IPMAC)
*----
*  RECOVER H-FACTORS
*----
      CALL SETARA(BASE,NMIX*NGRP,IHFAC)
      JPMAC=LCMGID(IPMAC,'GROUP')
      DO JGR=1,NGRP
      KPMAC=LCMGIL(JPMAC,JGR)
      CALL LCMLEN(KPMAC,'H-FACTOR',LENGT,ITYP)
      IF(LENGT.NE.NMIX)CALL XABORT('@MACINI: UNABLE TO FIND H'
     1 //'-FACTOR BLOCK DATA IN THE NEW MACROLIB.')
      CALL LCMGET(KPMAC,'H-FACTOR',BASE(IHFAC+(JGR-1)*NMIX))
      ENDDO
      CALL LCMPUT(IPMTX,'H-FACTOR',NMIX*NGRP,2,BASE(IHFAC))
      CALL RLSARA(BASE(IHFAC))
      RETURN
      END

==> version4_v4.0.7_lgpl.tgz: Version4_lgpl/Donjon/src/MACINI.f <==
*DECK MACINI
      SUBROUTINE MACINI(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* construct a new macrolib that will contain one mixture number per
* material region; fuel-map macrolib is required.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal.
*
*Author(s): J. Koclas, E. Varin, D. Sekki
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): create type(L_MACROLIB);
*         HENTRY(2): modification type(L_MATEX);
*         HENTRY(3): read-only type(L_MACROLIB);
*         HENTRY(4): read-only type(L_MACROLIB) (optional).
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NENTRY
      CHARACTER HENTRY(NENTRY)*12
      INTEGER IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40,IOUT=6)
      CHARACTER HSIGN*12,TEXT*12
      INTEGER IBASE(1),KCHAR(3),ISTATE(NSTATE)
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
      DOUBLE PRECISION DFLOT
      LOGICAL LMAP,LWD1,LWD2
*----
*  PARAMETER VALIDATION
*----
      IF((NENTRY.LE.2).OR.(NENTRY.GE.5)) 
     1  CALL XABORT('@MACINI: 4 PARAMETERS EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2))CALL XABORT('@RMA'
     1 //'C: LCM OBJECT EXPECTED AT LHS.')
      IF(JENTRY(1).NE.0)CALL XABORT('@MACINI: CREATE MODE EXPECTED'
     1 //' FOR L_MACROLIB AT LHS.')
      IPMAC=KENTRY(1)
*     L_MATEX
      IF((IENTRY(2).NE.1).AND.(IENTRY(2).NE.2))CALL XABORT('@RMA'
     1 //'C: LCM OBJECT EXPECTED AT RHS.')
      IF(JENTRY(2).NE.1)CALL XABORT('@MACINI: MODIFICATION MODE EX'
     1 //'PECTED FOR L_MATEX OBJECT.')
      CALL LCMGET(KENTRY(2),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
      IF(HSIGN.NE.'L_MATEX')THEN
        TEXT=HENTRY(2)
        CALL XABORT('@MACINI: SIGNATURE OF '//TEXT//' IS '//HSIGN//
     1  '. L_MATEX EXPECTED AT RHS.')
      ENDIF
      IPMTX=KENTRY(2)
      DO IEN=3,NENTRY
      IF((IENTRY(IEN).NE.1).AND.(IENTRY(IEN).NE.2))CALL XABORT('@R'
     1 //'MAC: LCM OBJECT EXPECTED AT RHS.')
      IF(JENTRY(IEN).NE.2)CALL XABORT('@MACINI: READ-ONLY MODE EXPEC'
     1 //'TED FOR THE LCM OBJECTS AT RHS.')
      ENDDO
*     L_MACROLIB(1)
      CALL LCMGET(KENTRY(3),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
      IF(HSIGN.NE.'L_MACROLIB')THEN
        TEXT=HENTRY(3)
        CALL XABORT('@MACINI: SIGNATURE OF '//TEXT//' IS '//HSIGN//
     1  '. FIRST MACROLIB EXPECTED AT RHS.')
      ENDIF
      IPMAC1=KENTRY(3)
*     L_MACROLIB(2)
      IF(NENTRY.EQ.4) THEN
      	CALL LCMGET(KENTRY(4),'SIGNATURE',KCHAR)
      	WRITE(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
      	IF(HSIGN.NE.'L_MACROLIB')THEN
          TEXT=HENTRY(4)
          CALL XABORT('@MACINI: SIGNATURE OF '//TEXT//' IS '//HSIGN//
     1    '. FUEL-MAP MACROLIB EXPECTED AT RHS.')
      	ENDIF
      	IPMAC2=KENTRY(4)
      ELSE
      	IPMAC2=0
      ENDIF
*----
*  RECOVER INFORMATION
*----
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPMAC1,'STATE-VECTOR',ISTATE)
*     MACROLIB(1)-INFO
      NGRP=ISTATE(1)
      NMIX1=ISTATE(2)
      NL=ISTATE(3)
      NDEL1=ISTATE(7)
      LEAK=ISTATE(9)
*     MACROLIB(2)-INFO
      IF(NENTRY.EQ.4) THEN
      	CALL XDISET(ISTATE,NSTATE,0)
      	CALL LCMGET(IPMAC2,'STATE-VECTOR',ISTATE)
      	NMIX2=ISTATE(2)
        NDEL2=ISTATE(7)
        NL=MAX(ISTATE(3),NL)
        IF(ISTATE(1).NE.NGRP)CALL XABORT('@MACINI: DIFFERENT NGR'
     1 //'P NUMBER IN TWO MACROLIB OBJECTS.')
      	IF(ISTATE(3).NE.NL)CALL XABORT('@MACINI: INCONSISTENT NL'
     1 //' NUMBER IN TWO MACROLIB OBJECTS.')
      	IF(ISTATE(9).NE.LEAK)CALL XABORT('@MACINI: DIFFERENT LEAK'
     1 //' NUMBER IN TWO MACROLIB OBJECTS.')
      ENDIF
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPMTX,'STATE-VECTOR',ISTATE)
*     MATEX-INFO
      NMIX=ISTATE(2)
      NTOT=ISTATE(5)
      CALL SETARA(IBASE,NTOT,IMIX)
      CALL XDISET(IBASE(IMIX),NTOT,0)
      CALL LCMGET(IPMTX,'MAT',IBASE(IMIX))
      IMPX=1
      CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(ITYP.EQ.10)GOTO 20
      IF(ITYP.NE.3)CALL XABORT('@MACINI: CHARACTER DATA EXPECTED.')
      IF(TEXT.NE.'EDIT')GOTO 10
*     READ PRINTING INDEX
      CALL REDGET(ITYP,IMPX,FLOT,TEXT,DFLOT)
      IF(ITYP.NE.1)CALL XABORT('@MACINI: INTEGER FOR EDIT EXPECTED.')
      CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
   10 IF(TEXT.NE.';')CALL XABORT('@MACINI: FINAL ; EXPECTED.')
*----
*  NEW MACROLIB CREATION
*----
   20 IF(IMPX.GT.1)WRITE(IOUT,*)'NUMBER OF ENERGY GROUPS  ',NGRP
      IF(IMPX.GT.1)WRITE(IOUT,*)'TOTAL NUMBER OF MIXTURES ',NMIX
*     DO NOT INCLUDE FUEL PROPERTIES
      LMAP=.FALSE.
      IF(IMPX.GT.0)WRITE(IOUT,*)'** TREATING FIRST MACROLIB **'
      CALL MACCRE(IPMAC1,IPMAC,NL,NGRP,NMIX1,NMIX,NTOT,
     1            IBASE(IMIX),LMAP,IMPX)
      IF(IMPX.GT.1)CALL LCMLIB(IPMAC)
*     INCLUDE FUEL PROPERTIES
      IF(NENTRY.EQ.4) THEN
      	LMAP=.TRUE.
      	IF(IMPX.GT.0)WRITE(IOUT,*)'** TREATING FUEL-MAP MACROLIB **'
      	CALL MACCRE(IPMAC2,IPMAC,NL,NGRP,NMIX2,NMIX,NTOT,
     1            IBASE(IMIX),LMAP,IMPX)
      	CALL RLSARA(IBASE(IMIX))
      ENDIF
*----
*  RECOVER LAMBDA-D
*----
      CALL LCMLEN(IPMAC1,'LAMBDA-D',LENGTH,ITYLCM)
      LWD1=(LENGTH.EQ.NDEL1).AND.(NDEL1.GT.0)
      CALL LCMLEN(IPMAC2,'LAMBDA-D',LENGTH,ITYLCM)
      LWD2=(LENGTH.EQ.NDEL2).AND.(NDEL2.GT.0)
      IF(LWD1) THEN
        NDEL=NDEL1
        CALL SETARA(BASE,NDEL,IWDLA)
        CALL LCMGET(IPMAC1,'LAMBDA-D',BASE(IWDLA))
        CALL LCMPUT(IPMAC,'LAMBDA-D',NDEL,2,BASE(IWDLA))
        CALL RLSARA(BASE(IWDLA))
      ELSE IF(LWD2) THEN
        NDEL=NDEL2
        CALL SETARA(BASE,NDEL,IWDLA)
        CALL LCMGET(IPMAC2,'LAMBDA-D',BASE(IWDLA))
        CALL LCMPUT(IPMAC,'LAMBDA-D',NDEL,2,BASE(IWDLA))
        CALL RLSARA(BASE(IWDLA))
      ENDIF
*----
*  STATE-VECTOR
*----
      CALL XDISET(ISTATE,NSTATE,0)
      ISTATE(1)=NGRP
      ISTATE(2)=NMIX
      ISTATE(3)=NL
      ISTATE(4)=1
      ISTATE(7)=NDEL
      ISTATE(9)=LEAK
      CALL LCMPUT(IPMAC,'STATE-VECTOR',NSTATE,1,ISTATE)
      HSIGN='L_MACROLIB'
      READ(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
      CALL LCMPUT(IPMAC,'SIGNATURE',3,3,KCHAR)
      IF(IMPX.GT.0)CALL LCMLIB(IPMAC)
*----
*  RECOVER H-FACTOR AND SAVE ON L_MATEX
*----
      CALL SETARA(BASE,NMIX*NGRP,IHFAC)
      JPMAC=LCMGID(IPMAC,'GROUP')
      DO JGR=1,NGRP
      KPMAC=LCMGIL(JPMAC,JGR)
      CALL LCMLEN(KPMAC,'H-FACTOR',LENGT,ITYP)
      IF(LENGT.NE.NMIX)CALL XABORT('@MACINI: UNABLE TO FIND H'
     1 //'-FACTOR BLOCK DATA IN THE NEW MACROLIB.')
      CALL LCMGET(KPMAC,'H-FACTOR',BASE(IHFAC+(JGR-1)*NMIX))
      ENDDO
      CALL LCMPUT(IPMTX,'H-FACTOR',NMIX*NGRP,2,BASE(IHFAC))
      CALL RLSARA(BASE(IHFAC))
      RETURN
      END
